#!/usr/bin/python -u
# vim: fileencoding=utf-8:nomodified:nowrap:textwidth=0:foldmethod=marker:foldcolumn=4:ruler:showcmd:lcs=tab\:|- list:tabstop=8:noexpandtab:nosmarttab:softtabstop=0:shiftwidth=0

import string

# Generate alphabet
alphabet = string.ascii_uppercase + string.ascii_lowercase + "0123456789#"

# Python script to generate test.ino for Arduino

fout="SpeedTestPgm_v.0.1.ino"
print(f"Generating {fout} file...")

with open(f"{fout}", 'w') as file:
	file.write(f''' /* vim: set noexpandtab fileencoding=utf-8 nomodified nowrap textwidth=270 foldmethod=marker foldmarker={{{{{{,}}}}}} foldcolumn=4 ruler showcmd lcs=tab\:|- list: */
// ,g = gcc, exactly one space after "set"
void setup() {{
	Serial.begin(115200);
	for (int i = 0; i < 2000; i++) {{Serial.print("=");}};	// fill buffers
	delay(1000);
	Serial.println();
	Serial.println("Testy {fout}:");
	for (int i = 0; i < 2000; i++) {{Serial.print("=");}};	// fill buffers
	unsigned long oldTime = millis();
''')
	t="Pgm_Print"	# {{{
	file.write(f''' // {{{{{{	Test_{t}
	unsigned long startTime_{t};
	Serial.println();
	Serial.println("{t}");
	while (oldTime == (startTime_{t} = millis())) {{}}; // wait for new millis
	for (int i = 0; i < 100; i++) {{
''')
	for i in range(0, 100):
		file.write(f'''		Serial.print(F("{alphabet[4*i % len(alphabet)]}{alphabet[(4*i + 1) % len(alphabet)]}{alphabet[(4*i + 2) % len(alphabet)]}{alphabet[(4*i + 3) % len(alphabet)]}{"12%02d"%i}"));
''')
	file.write(f'''	}}
	unsigned long endTime_{t} = millis();
// }}}}}}	Test_{t} end

''')
	# }}}
	t="Raw_Print"	# {{{
	file.write(f''' // {{{{{{	Test_{t}
	unsigned long startTime_{t};
	Serial.println();
	Serial.println("{t}");
	while (oldTime == (startTime_{t} = millis())) {{}}; // wait for new millis
	for (int i = 0; i < 100; i++) {{
''')
	for i in range(0, 100):
		file.write(f'''		Serial.print("{alphabet[4*i % len(alphabet)]}{alphabet[(4*i + 1) % len(alphabet)]}{alphabet[(4*i + 2) % len(alphabet)]}{alphabet[(4*i + 3) % len(alphabet)]}{"21%02d"%i}");
''')
	file.write(f'''	}}
	unsigned long endTime_{t} = millis();
// }}}}}}	Test_{t} end

''')
	# }}}
	t="Pgm_Cnt"	# {{{ 
	file.write(f''' // {{{{{{	Test_{t}
''')
	for i in range(0, 100):
		file.write(f'''	volatile const static char tmp_{t}_{"%02d"%i}[] PROGMEM = {{"{alphabet[4*i % len(alphabet)]}{alphabet[(4*i + 1) % len(alphabet)]}{alphabet[(4*i + 2) % len(alphabet)]}{alphabet[(4*i + 3) % len(alphabet)]}{"34%02d"%i}"}};
''')
	file.write(f'''
	unsigned long tmp_{t} = 0;
	unsigned long startTime_{t};
	Serial.println();
	Serial.println("{t}");
	while (oldTime == (startTime_{t} = millis())) {{}}; // wait for new millis
	for (int i = 0; i < 100; i++) {{
''')
	for i in range(0, 100):
		for j in range(0, 8):
			file.write(f'''		tmp_{t}+=(byte)pgm_read_byte_near(tmp_{t}_{"%02d"%i}+{j});
''')
	file.write(f'''	}}
	unsigned long endTime_{t} = millis();
	Serial.print("tmp_{t}:");
	Serial.println(tmp_{t});
// }}}}}}	Test_{t} end

''')
	# }}}
	t="Raw_Cnt"	# {{{
	file.write(f''' // {{{{{{	Test_{t}
''')
	for i in range(0, 100):
		file.write(f'''	volatile const static char tmp_{t}_{"%02d"%i}[]="{alphabet[4*i % len(alphabet)]}{alphabet[(4*i + 1) % len(alphabet)]}{alphabet[(4*i + 2) % len(alphabet)]}{alphabet[(4*i + 3) % len(alphabet)]}{"43%02d"%i}";
''')
	file.write(f'''
	unsigned long tmp_{t} = 0;
	unsigned long startTime_{t};
	Serial.println();
	Serial.println("{t}");
	while (oldTime == (startTime_{t} = millis())) {{}}; // wait for new millis
	for (int i = 0; i < 100; i++) {{
''')
	for i in range(0, 100):
		for j in range(0, 8):
			file.write(f'''		tmp_{t}+=(byte)tmp_{t}_{"%02d"%i}[{j}];
''')
	file.write(f'''	}}
	unsigned long endTime_{t} = millis();
	Serial.print("tmp_{t}:");
	Serial.println(tmp_{t});
// }}}}}}	Test_{t} end

''')
	# }}}
	file.write(f'''
	Serial.println();
	Serial.println();
''')
	for t in ("Pgm_Print","Raw_Print","Pgm_Cnt","Raw_Cnt"):
		file.write(f'''
	Serial.print("Test_{t}: ");
	Serial.println(endTime_{t} - startTime_{t});
''')
	file.write('''
// The end
}

void loop() {
}
''')


print(f"{fout} file generated successfully.")
